#include <avr/io.h>

#include <stdbool.h>

// The UART is on PD2 (Rx) and PD3 (Tx)
// The gamepads are all connected on port B
#define PCW_PORT PORTD
#define PCW_DDR  DDRD
#define PCW_PIN  PIND
#define PCW_CLK_A  (1 << 0)
#define PCW_DAT_A  (1 << 1)

static uint8_t data;
static uint8_t address;

static uint8_t oldv;
static uint8_t newv;

static uint8_t pulses;

static uint8_t kmatrix[16];

uint8_t scanbyte()
{
	pulses = 0;
	address = 0;

	oldv = PCW_PIN;

	for (int i = 0; i < 2048; i++) {
		newv = PCW_PIN;

		if (pulses < 2) {
			// Initial sync phase: 2 pulses on data without changes of clock
			if (((newv & PCW_CLK_A) == 0) && ((oldv & PCW_CLK_A) != 0)) {
				// Clock moved while we were waiting for the initial data pulses
				// We are not in sync, start over
				pulses = 0;
				address = 0;
			}
			if (((newv & PCW_DAT_A) == 0) && ((oldv & PCW_DAT_A) != 0)) {
				pulses++;
			}
		} else if (pulses < 6) {
			// Receiving address (4 bits)
			if (((newv & PCW_CLK_A) == 0) && ((oldv & PCW_CLK_A) != 0)) {
				address = (address << 1) | ((newv & PCW_DAT_A) ? 1 : 0);
				pulses++;
			}
		} else {
			// Receiving data (8 bits)
			if (((newv & PCW_CLK_A) == 0) && ((oldv & PCW_CLK_A) != 0)) {
				data = (data << 1) | ((newv & PCW_DAT_A) ? 1 : 0);
				pulses++;
			}

			// If we reached the last bit, we can stop
			if (pulses >= 10) {
				kmatrix[address] = data;
				return address;
			}
		}

		oldv = newv;
	}

	// TODO set up some error LED or something?
	return 255;
}

extern "C" uint8_t* scanall()
{
	uint8_t address;

	do {
		address = scanbyte();
	} while ((address < 15) && (kmatrix[15] & 0x80 != 0));

	return kmatrix;
}

extern "C" void PCW_Init() {
	// GrIP input
	PCW_PORT = 0xFF; // Enable pull-ups
	PCW_DDR = 0;  // Port as input
}

